@@ -349,6 +349,6 @@ periodic_policy_dynamic<-function (demand,forecast, leadtime, Review_period,serv
 
 periodic_policy_dynamic(demand = rpois(90,9),forecast = rpois(90,9),
                         service_level = 0.9,leadtime = 10,distribution = 'gamma',
-                        Review_period = 10,smoothing_error = 0.1,Backlogs = TRUE)                       
+                        Review_period = 10,smoothing_error = 0.1,Backlogs = TRUE,plot = TRUE)                       
                         
                       
